package net.wanji.web.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import net.wanji.feign.pojo.result.Result;
import net.wanji.feign.pojo.vo.*;
import net.wanji.feign.service.UtcFeignClients;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/01/04 16:10
 */
@Api(value = "SignalServiceController", description = "信号控制")
@RequestMapping("/controlCommand")
@RestController
@SuppressWarnings("all")
public class SignalServiceController {

    @Autowired
    private UtcFeignClients utcFeignClients;

    @ApiOperation(value = "信号机方案下发-基础方案下发", notes = "信号机方案下发-基础方案下发")
    @PostMapping("/schemeSend")
    public <T> Result<T> schemeSend(@RequestBody SchemeSendVO schemeSendVO) {
        Result result = utcFeignClients.schemeSend(schemeSendVO);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "信号机方案下发-基础方案下发UTC服务调用异常");
        }
        return Result.success("success");
    }

    @ApiOperation(value = "计划下发-计划信息、时段信息", notes = "计划下发-计划信息、时段信息")
    @PostMapping("/planSend")
    public <T> Result<T> planSend(@RequestBody PlanSendVO planSendVO) throws Exception {
        Result result = utcFeignClients.planSend(planSendVO);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "计划下发-计划信息、时段信息UTC服务调用异常");
        }
        return Result.success("success");
    }

    @ApiOperation(value = "时间表下发-时间表信息", notes = "时间表下发-时间表信息")
    @PostMapping("/scheduleSend")
    public <T> Result<T> scheduleSend(@RequestBody ScheduleSendVO scheduleSendVO) throws Exception {
        Result result = utcFeignClients.scheduleSend(scheduleSendVO);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "时间表下发-时间表信息UTC服务调用异常");
        }
        return Result.success("success");
    }

    @ApiOperation(value = "相位配时下发-（下发相位参数）", notes = "相位配时下发-（下发相位参数）")
    @PostMapping("/phaseTimingSend")
    public <T> Result<T> phaseTimingSend(@RequestBody PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        Result result = utcFeignClients.phaseTimingSend(phaseTimingSendVO);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "相位配时下发-（下发相位参数）UTC服务调用异常");
        }
        return Result.success("success");
    }

    @ApiOperation(value = "灯态控制-路口全红/黄闪/关灯控制/恢复", notes = "灯态控制-路口全红/黄闪/关灯控制/恢复")
    @PostMapping("/lampStateControl")
    public <T> Result<T> allRedControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        Integer controlType = commandVO.getControlType();
        Result result = new Result();
        if (1 == controlType) {
            result = utcFeignClients.allRedControl(commandVO);
        }
        if (2 == controlType) {
            result = utcFeignClients.yellowLightControl(commandVO);
        }
        if (3 == controlType) {
            result = utcFeignClients.closeLightControl(commandVO);
        }
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "灯态控制UTC服务调用异常");
        }
        return Result.success("success");
    }


    @ApiOperation(value = "步进控制-步进控制/恢复", notes = "步进控制-步进控制/恢复")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "command", value = "1 开始步进 0 取消步进", paramType = "query", required = true, dataType = "int"),
            @ApiImplicitParam(name = "stepNum", value = "0 顺序步进 n 跳过n个相位", paramType = "query", required = true, dataType = "int"),
    })
    @PostMapping("/stepControl")
    public <T> Result<T> stepControl(@RequestParam String code,
                                     @RequestParam Integer command,
                                     @RequestParam int stepNum) throws Exception {
        Result result = utcFeignClients.stepControl(code, command, stepNum);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "步进控制-步进控制/恢复UTC服务调用异常");
        }
        return Result.success("success");
    }

    @ApiOperation(value = "恢复时间表-恢复路口时间表执行", notes = "恢复时间表-恢复路口时间表执行")
    @PostMapping("/recoverSchedule")
    public <T> Result<T> recoverSchedule(@RequestParam String code) throws Exception {
        Result result = utcFeignClients.recoverSchedule(code);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "恢复时间表-恢复路口时间表执行UTC服务调用异常");
        }
        return Result.success("success");
    }

    @ApiOperation(value = "锁定控制-相位锁定/解锁", notes = "锁定控制-相位锁定/解锁")
    @PostMapping("/lockControl")
    public <T> Result<T> lockControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        Result result = utcFeignClients.lockControl(commandVO);
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "锁定控制-相位锁定/解锁UTC服务调用异常");
        }
        return Result.success("success");
    }

    @ApiOperation(value = "相位差下发", notes = "相位差下发")
    @PostMapping("/phaseDiffSend")
    public <T> Result<T> phaseDiffSend(HttpServletRequest httpServletRequest) {
        Result result = utcFeignClients.phaseDiffSend();
        if (Objects.isNull(result) || result.getState() != 200) {
            return Result.error(result.getState(), "相位差下发UTC服务调用异常");
        }
        return Result.success("success");
    }

}
