package net.wanji.web.service.scheme.impl;

import net.wanji.web.common.constant.ControlModeConst;
import net.wanji.web.common.exception.PhaseNoNotUniqueException;
import net.wanji.web.dto.LaneIdDTO;
import net.wanji.web.dto.LightIdDTO;
import net.wanji.web.dto.SaveSchemeConfigDTO;
import net.wanji.web.mapper.scheme.CrossAccompanyPhaseMapper;
import net.wanji.web.mapper.scheme.CrossLaneLightsMapper;
import net.wanji.web.mapper.scheme.CrossLightsMapper;
import net.wanji.web.mapper.scheme.CrossPhaseLightsMapper;
import net.wanji.web.mapper.scheme.CrossPhaseMapper;
import net.wanji.web.mapper.scheme.CrossSchemeMapper;
import net.wanji.web.mapper.scheme.LaneInfoMapper;
import net.wanji.web.po.scheme.CrossAccompanyPhasePO;
import net.wanji.web.po.scheme.CrossLaneLightsPO;
import net.wanji.web.po.scheme.CrossLightsPO;
import net.wanji.web.po.scheme.CrossPhaseLightsPO;
import net.wanji.web.po.scheme.CrossPhasePO;
import net.wanji.web.po.scheme.CrossSchemePO;
import net.wanji.web.service.scheme.SchemeConfigService;
import net.wanji.web.vo.scheme.LaneIdsVO;
import net.wanji.web.vo.scheme.LightIdVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/1/5 10:27
 */
@Service
public class SchemeConfigServiceImpl implements SchemeConfigService {
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final CrossLaneLightsMapper crossLaneLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossAccompanyPhaseMapper crossAccompanyPhaseMapper;

    public SchemeConfigServiceImpl(CrossSchemeMapper crossSchemeMapper, CrossPhaseMapper crossPhaseMapper,
                                   LaneInfoMapper laneInfoMapper, CrossLaneLightsMapper crossLaneLightsMapper,
                                   CrossPhaseLightsMapper crossPhaseLightsMapper, CrossLightsMapper crossLightsMapper, CrossAccompanyPhaseMapper crossAccompanyPhaseMapper) {
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossPhaseMapper = crossPhaseMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.crossLaneLightsMapper = crossLaneLightsMapper;
        this.crossPhaseLightsMapper = crossPhaseLightsMapper;
        this.crossLightsMapper = crossLightsMapper;
        this.crossAccompanyPhaseMapper = crossAccompanyPhaseMapper;
    }

    @Override
    @Transactional
    public void saveSchemeConfig(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        Integer timeType = saveSchemeConfigDTO.getTimeType();
        if (timeType == 2) { //环式
            handleRingTime(saveSchemeConfigDTO);
        } else if (timeType == 1) { //阶段式
            // TODO: 2023/1/6 处理阶段式
        }
    }

    @Override
    public LightIdVO getLightByLane(LaneIdDTO laneIdDTO) {
        String crossId = laneIdDTO.getCrossId();
        String laneId = laneIdDTO.getLaneId();
        Integer lightId = crossLaneLightsMapper.selectLightsIdByLaneId(crossId, laneId);
        LightIdVO lightIdVO = new LightIdVO();
        lightIdVO.setCrossId(crossId);
        lightIdVO.setLightId(lightId);
        return lightIdVO;
    }

    @Override
    public LaneIdsVO getLanesByLight(LightIdDTO lightIdDTO) {
        String crossId = lightIdDTO.getCrossId();
        LaneIdsVO laneIdsVO = new LaneIdsVO();
        laneIdsVO.setCrossId(crossId);

        Integer lightId = lightIdDTO.getLightId();
        List<CrossLaneLightsPO> crossLaneLightsPOList = crossLaneLightsMapper.selectLanesByLightId(crossId, lightId);
        List<String> laneIds = crossLaneLightsPOList.stream()
                .map(CrossLaneLightsPO::getLaneId)
                .collect(Collectors.toList());
        laneIdsVO.setLaneIds(laneIds);

        return laneIdsVO;
    }

    private void handleRingTime(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        // 更新方案基础信息表
        updateCrossScheme(saveSchemeConfigDTO);
        // 更新相位基础表、相位灯组关系表、伴随相位母相位关系
        updateCrossPhase(saveSchemeConfigDTO);
        // 更新协调相位ID
        updateCoordPhase(saveSchemeConfigDTO);
    }

    private void updateCoordPhase(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            String schemeNo = phaseScheme.getSchemeNo();
            Integer schemeId = crossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
            String coordPhaseNo = phaseScheme.getCoordPhaseNo();
            Integer phaseId = crossPhaseMapper.selectIdByCrossIdAndPhaseNo(crossId, schemeId, coordPhaseNo);
            crossSchemeMapper.updateCoordPhase(crossId, schemeNo, phaseId);
        }
    }

    private void updateCrossPhase(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        crossPhaseMapper.deleteByCrossId(crossId);
        crossPhaseLightsMapper.deleteByCrossId(crossId);
        crossAccompanyPhaseMapper.deleteByCrossId(crossId);

        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            // 用于限制相位号不能重复
            Set<String> phaseNoSet = new HashSet<>();
            String schemeNo = phaseScheme.getSchemeNo();
            // 获取方案ID
            Integer schemeId = crossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
            List<SaveSchemeConfigDTO.RingListElement> ringList = phaseScheme.getRingList();
            // 用于维护伴随相位
            Map<String, SaveSchemeConfigDTO.PhaseListElement> phaseNoMap = new HashMap<>();
            for (SaveSchemeConfigDTO.RingListElement ringListElement : ringList) {
                Integer ringNo = ringListElement.getRingNo();
                List<SaveSchemeConfigDTO.GroupListElement> groupList = ringListElement.getGroupList();
                for (SaveSchemeConfigDTO.GroupListElement groupListElement : groupList) {
                    Integer groupNo = groupListElement.getGroupNo();
                    List<SaveSchemeConfigDTO.PhaseListElement> phaseList = groupListElement.getPhaseList();
                    for (int i = 0; i < phaseList.size(); i++) {
                        SaveSchemeConfigDTO.PhaseListElement phase = phaseList.get(i);
                        String phaseNo = phase.getPhaseNo();
                        testPhaseNo(phaseNoSet, phaseNo);
                        Integer phaseType = phase.getPhaseType();
                        if (phaseType != 3) { // 实相位或虚相位
                            phaseNoMap.put(phaseNo, phase);
                            Integer yellowFlashTime = phase.getYellowFlashTime();
                            Integer redFlashTime = phase.getRedFlashTime();
                            Integer greenTime = phase.getGreenTime();
                            Integer greenFlashTime = phase.getGreenFlashTime();
                            Integer yellowTime = phase.getYellowTime();
                            Integer redTime = phase.getRedTime();
                            Integer minGreenTime = phase.getMinGreenTime();
                            Integer maxGreenTime = phase.getMaxGreenTime();

                            CrossPhasePO crossPhasePO = new CrossPhasePO();
                            crossPhasePO.setPhaseNo(phaseNo);
                            crossPhasePO.setPhaseType(phaseType);
                            crossPhasePO.setSort(i + 1);
                            crossPhasePO.setCrossId(crossId);
                            crossPhasePO.setPlanId(schemeId);
                            crossPhasePO.setRingNo(ringNo);
                            crossPhasePO.setGroupNo(groupNo);
                            // 控制模式
                            crossPhasePO.setControlMode(ControlModeConst.FIXED_CYCLE);

                            crossPhasePO.setGreenTime(greenTime);
                            crossPhasePO.setGreenFlashTime(greenFlashTime);
                            crossPhasePO.setYellowTime(yellowTime);
                            crossPhasePO.setRedTime(redTime);
                            crossPhasePO.setYellowFlashTime(yellowFlashTime);
                            crossPhasePO.setRedFlashTime(redFlashTime);
                            crossPhasePO.setMinGreenTime(minGreenTime);
                            crossPhasePO.setMaxGreenTime(maxGreenTime);
                            crossPhasePO.setPhaseTime(greenTime + greenFlashTime + yellowTime + redTime
                                    + yellowFlashTime + redFlashTime);
                            // 插入相位表记录，获取ID
                            crossPhaseMapper.insertOne(crossPhasePO);
                            Integer phaseId = crossPhasePO.getId();
                            if (phaseType == 1) { // 实相位
                                // 更新相位灯组关系表
                                updateCrossPhaseLights(crossId, phase, phaseId);
                            }
                        } else { // 伴随相位
                            CrossPhasePO crossPhasePO = new CrossPhasePO();
                            crossPhasePO.setPhaseNo(phaseNo);
                            crossPhasePO.setPhaseType(phaseType);
                            crossPhasePO.setSort(i + 1);
                            crossPhasePO.setCrossId(crossId);
                            crossPhasePO.setPlanId(schemeId);
                            crossPhasePO.setRingNo(ringNo);
                            crossPhasePO.setGroupNo(groupNo);
                            // 控制模式
                            crossPhasePO.setControlMode(ControlModeConst.FIXED_CYCLE);
                            Integer coordGreenTime = 0;
                            Integer coordGreenFlashTime = 0;
                            Integer coordYelloFlashTime = 0;
                            Integer coordRedFlashTime = 0;
                            Integer coordYellowTime = 0;
                            Integer coordRedTime = 0;
                            Integer coordMinGreenTime = 0;
                            Integer coordMaxGreenTime = 0;
                            List<String> superPhaseNoList = phase.getAccompanyPhaseNoList();
                            if (superPhaseNoList != null) {
                                for (String superPhaseNo : superPhaseNoList) {
                                    SaveSchemeConfigDTO.PhaseListElement phaseListElement = phaseNoMap.get(superPhaseNo);
                                    // 时长加总
                                    coordGreenTime += phaseListElement.getGreenTime();
                                    coordGreenFlashTime += phaseListElement.getGreenFlashTime();
                                    coordYelloFlashTime += phaseListElement.getYellowFlashTime();
                                    coordRedFlashTime += phaseListElement.getRedFlashTime();
                                    coordYellowTime += phaseListElement.getYellowTime();
                                    coordRedTime += phaseListElement.getRedTime();
                                    coordMinGreenTime += phaseListElement.getMinGreenTime();
                                    coordMaxGreenTime += phaseListElement.getMaxGreenTime();
                                }
                                crossPhasePO.setGreenTime(coordGreenTime);
                                crossPhasePO.setGreenFlashTime(coordGreenFlashTime);
                                crossPhasePO.setYellowFlashTime(coordYelloFlashTime);
                                crossPhasePO.setRedFlashTime(coordRedFlashTime);
                                crossPhasePO.setYellowTime(coordYellowTime);
                                crossPhasePO.setRedTime(coordRedTime);
                                crossPhasePO.setMinGreenTime(coordMinGreenTime);
                                crossPhasePO.setMaxGreenTime(coordMaxGreenTime);
                                crossPhasePO.setPhaseTime(coordGreenTime + coordGreenFlashTime + coordYelloFlashTime
                                        + coordRedFlashTime + coordYellowTime + coordRedTime);
                                // 插入相位表记录获取ID
                                crossPhaseMapper.insertOne(crossPhasePO);
                                Integer accompanyPhaseId = crossPhasePO.getId();

                                // 更新伴随相位母相位关系表
                                List<CrossAccompanyPhasePO> crossAccompanyPhasePOList = new ArrayList<>();
                                for (String superPhaseNo : superPhaseNoList) {
                                    CrossAccompanyPhasePO crossAccompanyPhasePO = new CrossAccompanyPhasePO();
                                    crossAccompanyPhasePO.setCrossId(crossId);
                                    crossAccompanyPhasePO.setPhaseId(accompanyPhaseId);
                                    Integer superPhaseId = crossPhaseMapper.selectIdByCrossIdAndPhaseNo(
                                            crossId, schemeId,superPhaseNo);
                                    crossAccompanyPhasePO.setSuperId(superPhaseId);
                                    crossAccompanyPhasePOList.add(crossAccompanyPhasePO);
                                }
                                crossAccompanyPhaseMapper.insertBatch(crossAccompanyPhasePOList);
                            }
                        }
                    }
                }
            }
            // 处理特殊控制
            List<SaveSchemeConfigDTO.PhaseListElement> specialControlModeList = phaseScheme.getSpecialControlModeList();
            if (specialControlModeList != null) {
                for (int i = 0; i < specialControlModeList.size(); i++) {
                    SaveSchemeConfigDTO.PhaseListElement phase = specialControlModeList.get(i);
                    String phaseNo = phase.getPhaseNo();
                    testPhaseNo(phaseNoSet, phaseNo);
                    CrossPhasePO crossPhasePO = new CrossPhasePO();
                    crossPhasePO.setPhaseNo(phase.getPhaseNo());
                    crossPhasePO.setSort(i + 1);
                    crossPhasePO.setCrossId(crossId);
                    crossPhasePO.setPlanId(schemeId);
                    crossPhasePO.setPhaseType(1);
                    // 控制模式
                    if (phase.getYellowFlash() == 1) {
                        crossPhasePO.setControlMode(7);
                    } else if (phase.getIsRed() == 1) {
                        crossPhasePO.setControlMode(4);
                    } else if (phase.getIsGreen() == 1) {
                        crossPhasePO.setControlMode(2);
                    } else if (phase.getGreenFlash() == 1) {
                        crossPhasePO.setControlMode(6);
                    }
                    // 插入相位表记录，获取ID
                    crossPhaseMapper.insertOne(crossPhasePO);
                    Integer phaseId = crossPhasePO.getId();
                    // 更新相位灯组关系表
                    updateCrossPhaseLights(crossId, phase, phaseId);
                }
            }
        }
    }

    private static void testPhaseNo(Set<String> phaseNoSet, String phaseNo) throws PhaseNoNotUniqueException {
        boolean isUnique = phaseNoSet.add(phaseNo);
        if (!isUnique) {
            throw new PhaseNoNotUniqueException("同一方案中相位号不可重复");
        }
    }


    private void updateCrossPhaseLights(String crossId,
                                        SaveSchemeConfigDTO.PhaseListElement phase,
                                        Integer phaseId) {
        Set<CrossPhaseLightsPO> crossPhaseLightsPOSet = new HashSet<>();
        List<SaveSchemeConfigDTO.DirListElement> dirList = phase.getDirList();
        for (SaveSchemeConfigDTO.DirListElement dirListElement : dirList) {
            Integer dir = dirListElement.getDir();
            // 更新行人相位-灯组关系
            Integer hasPersonPhase = dirListElement.getHasPersonPhase();
            if (hasPersonPhase == 1) {
                List<CrossLightsPO> crossLightsPOList = crossLightsMapper.selectByCrossIdAndDir(crossId, dir);
                for (CrossLightsPO crossLightsPO : crossLightsPOList) {
                    Integer type = crossLightsPO.getType();
                    if (type >= 20 && type < 30) { // 行人灯
                        Integer lightId = crossLightsPO.getId();
                        CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
                        crossPhaseLightsPO.setCrossId(crossId);
                        crossPhaseLightsPO.setLightsId(lightId);
                        crossPhaseLightsPO.setPhaseId(phaseId);
                        crossPhaseLightsPOSet.add(crossPhaseLightsPO);
                    }
                }
            }
            List<SaveSchemeConfigDTO.LaneListElement> laneList = dirListElement.getLaneList();
            for (SaveSchemeConfigDTO.LaneListElement laneListElement : laneList) {
                CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
                Integer category = laneListElement.getLaneType();
                Integer turn = laneListElement.getDirection();
                // 根据车道获得灯组ID
                String laneId = laneInfoMapper.selectIdByCrossIdDirTurn(crossId, dir, turn, category);
                Integer lightsId = crossLaneLightsMapper.selectLightsIdByLaneId(crossId, laneId);
                if (lightsId != null) {
                    crossPhaseLightsPO.setLightsId(lightsId);
                }
                crossPhaseLightsPO.setPhaseId(phaseId);
                crossPhaseLightsPO.setCrossId(crossId);
                crossPhaseLightsPOSet.add(crossPhaseLightsPO);
            }
        }
        crossPhaseLightsMapper.insertBatchBySet(crossPhaseLightsPOSet);
    }

    private void updateCrossScheme(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        crossSchemeMapper.deleteByCrossId(crossId);
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        List<CrossSchemePO> crossSchemePOList = new ArrayList<>();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            CrossSchemePO crossSchemePO = new CrossSchemePO();
            String schemeNo = phaseScheme.getSchemeNo();// 方案号
            crossSchemePO.setSchemeNo(schemeNo);
            String name = phaseScheme.getName(); // 方案名称
            crossSchemePO.setName(name);
            crossSchemePO.setCrossId(crossId);
            crossSchemePO.setCycle(phaseScheme.getCycle());
            crossSchemePO.setOffset(phaseScheme.getOffset());
            crossSchemePO.setSource(2); // 来自平台
            crossSchemePO.setIsDeleted(0);
            crossSchemePOList.add(crossSchemePO);
        }
        crossSchemeMapper.insertBatch(crossSchemePOList);
    }
}
